Attribute VB_Name = "modChol"
'
'    Copyright (C) 2008  Colin Caprani - www.colincaprani.com
'
'    This program is free software: you can redistribute it and/or modify
'    it under the terms of the GNU General Public License as published by
'    the Free Software Foundation, either version 3 of the License, or
'    (at your option) any later version.
'
'    This program is distributed in the hope that it will be useful,
'    but WITHOUT ANY WARRANTY; without even the implied warranty of
'    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
'    GNU General Public License for more details.
'
'    You should have received a copy of the GNU General Public License
'    along with this program.  If not, see <http://www.gnu.org/licenses/>.

Option Explicit
Dim p() As Double, neqn As Integer, InputMatrix() As Double

Sub Chol_DC(InputMatrix() As Double, n As Integer)
Dim i As Integer, j As Integer, k As Integer
Dim sum As Double

Call initVariables(n)

For i = 1 To n
    For j = i To n
        sum = InputMatrix(i, j)
        For k = i - 1 To 1 Step -1
            sum = sum - InputMatrix(i, k) * InputMatrix(j, k)
        Next k
        If i = j Then
            If sum < 0 Then
                MsgBox "Cholesky Decomposition Failed"
                Exit Sub
            Else
                p(i) = sum ^ 0.5
            End If
        Else
            InputMatrix(j, i) = sum / p(i)
        End If
    Next j
Next i

Call copyData(InputMatrix())

End Sub

Sub Chol_BS(b() As Double, x() As Double)
Dim n As Integer, i As Integer, j As Integer, k As Integer
Dim sum As Double

On Error GoTo errorhandle
'Solve Ly = b for y
For i = 1 To neqn
    sum = b(i)
    For k = i - 1 To 1 Step -1
        sum = sum - InputMatrix(i, k) * x(k)
    Next k
    x(i) = sum / p(i)
Next i

'Solve LT x = y for x
For i = neqn To 1 Step -1
    sum = x(i)
    For k = i + 1 To neqn
        sum = sum - InputMatrix(k, i) * x(k)
    Next k
    x(i) = sum / p(i)
Next i

Exit Sub
errorhandle:
Exit Sub
End Sub

Private Sub copyData(matrix() As Double)
Dim i As Integer, j As Integer

For i = 1 To neqn
    For j = 1 To neqn
        InputMatrix(i, j) = matrix(i, j)
    Next j
Next i
    
End Sub

Private Sub initVariables(n As Integer)
neqn = n
ReDim p(neqn)
ReDim InputMatrix(neqn, neqn)
End Sub
